import random
from userprofile_app.models import Profile
from rest_framework import serializers
from .models import City, CustomUser, OTPVerification, PrivacySettings, State, StateShortNames
from django.contrib.auth import get_user_model
from rest_framework import serializers
from .models import CustomUser
from .models import OnboardingImage


class OnboardingImageSerializer(serializers.ModelSerializer):
    class Meta:
        model = OnboardingImage
        fields = ['id', 'title', 'short_description', 'image']



class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = CustomUser
        fields = ['username', 'mobile_or_email', 'password']
        extra_kwargs = {'password': {'write_only': True}}

    def create(self, validated_data):
        user = CustomUser(**validated_data)
        user.set_password(validated_data['password'])
        user.save()
        return user

class RegisterSerializer(serializers.ModelSerializer):
    class Meta:
        model = CustomUser
        fields = ['full_name', 'mobile_or_email', 'password', 'login_method','device_type']
        extra_kwargs = {'password': {'write_only': True}}

    def create(self, validated_data):        
        user = CustomUser(**validated_data)
        user.set_password(validated_data['password'])
        user.save()
        return user

class OTPSerializer(serializers.ModelSerializer):
    class Meta:
        model = OTPVerification
        fields = ['user','mobile_or_email', 'otp', 'otp_expires_at', 'is_verified']


class ResetPasswordSerializer(serializers.Serializer):
    mobile_or_email = serializers.EmailField()

    def validate_mobile_or_email(self, value):
        # Check if the user exists with the given mobile or email
        User = get_user_model()
        if not User.objects.filter(mobile_or_email=value).exists():
            raise serializers.ValidationError("User does not exist.")
        return value

class ChangePasswordSerializer(serializers.Serializer):
    current_password = serializers.CharField(required=True, write_only=True)
    new_password = serializers.CharField(required=True, write_only=True)
    confirm_password = serializers.CharField(required=True, write_only=True)

    def validate_new_password(self, value):
        # Add custom password validation if needed (e.g., length, complexity)
        if len(value) < 8:
            raise serializers.ValidationError("New password must be at least 8 characters long.")
        return value


class LogoutSerializer(serializers.Serializer):
    refresh_token = serializers.CharField(required=True)

class CitySerializer(serializers.ModelSerializer):
    class Meta:
        model = City
        fields = ['id', 'name']

class StateSerializer(serializers.ModelSerializer):
    cities = CitySerializer(many=True, read_only=True)

    class Meta:
        model = State
        fields = ['id', 'name', 'cities']

class StateShortNamesSerializer(serializers.ModelSerializer):
    class Meta:
        model = StateShortNames
        fields = ['id', 'name', 'code']

class CustomUserSerializer(serializers.ModelSerializer):
    profile_image = serializers.SerializerMethodField()
    selected_industry = serializers.SerializerMethodField()
    age = serializers.SerializerMethodField()
    # location = serializers.SerializerMethodField()
    class Meta:
        model = CustomUser
        fields = (
            'id',
            'username', 
            'age',
            # 'location',
            'mobile_or_email', 
            'full_name',
            'device_type',
            'device_model',
            'user_status',
            'is_verified',
            'is_active',
            'is_staff',
            'profile_image', 
            'selected_industry',
        )

    def get_profile_image(self, obj):
        # Fetch the related Profile object and return the profile_image
        profile = Profile.objects.filter(user=obj).first()
        if profile:
            return profile.profile_image.url if profile.profile_image else None
        return None
    
    def get_age(self, obj):
        profile = Profile.objects.filter(user=obj).first()
        if profile:
            return profile.age if profile.age else None
        return None
    
    # def get_location(self, obj):
    #     profile = Profile.objects.filter(user=obj.id).first()

    #     if profile.location:
    #         parts = profile.location.split(',')
    #         if len(parts) == 2:
    #             city_id, state_id = map(int, parts)  # Convert both parts to integers

    #             # Fetch city and state names
    #             city = City.objects.filter(id=city_id).first()
    #             state = State.objects.filter(id=state_id).first()

    #             if city and state:
    #                 location = f"{city.name}, {state.name}"
    #             else:
    #                 location = None
        # return None


    def get_selected_industry(self, obj):
        # Fetch the related Profile object and return the selected_industry
        profile = Profile.objects.filter(user=obj).first()
        if profile:
            return profile.selected_primary_industry.name if profile.selected_primary_industry else None
        return None
    
class PrivacySettingsSerializer(serializers.ModelSerializer):
    class Meta:
        model = PrivacySettings
        fields = [
            'enable_notifications', 
            'account_notifications', 
            'promotions_email', 
            'promotions_sms', 
            'promotions_whatsapp', 
            'app_updates'
        ]